<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

if (!class_exists('WC_Payment_Gateway')) {
    return;
}

class WC_Gateway_Pay2S extends WC_Payment_Gateway {

    public function __construct() {
        // Định nghĩa URL của plugin
        if (!defined('WOO_PAY2S_PLUGIN_URL')) {
            define('WOO_PAY2S_PLUGIN_URL', plugin_dir_url(__FILE__));
        }
        
        $this->id = 'pay2s';
       // $this->icon = $this->get_option('bankIcon') ?: apply_filters('woocommerce_bacs_icon', plugin_dir_url(__DIR__).'assets/images/bank.svg');
        $this->has_fields = false;
        $this->method_title = __('Pay2S', 'woo-pay2s');
        $this->method_description = __('Tích hợp cổng thanh toán Pay2S với WooCommerce.', 'woo-pay2s');

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();
        
        $this->bankAccounts = get_option('woocommerce_bacs_accounts', [
            [
                'account_number' => $this->get_option('account_number'),
                'bank_id' => $this->get_option('bank_id'),
            ],
        ]);

        // Define user settings
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->order_button_text = $this->get_option('button_label');
        $this->enabled = $this->get_option('enabled');
        $this->partner_code = $this->get_option('partner_code');
        $this->access_key = $this->get_option('access_key');
        $this->secret_key = $this->get_option('secret_key');
        $this->endpoint = $this->get_option('api_endpoint');
        $this->order_desc = $this->get_option('order_desc');
        $this->order_created = $this->get_option('order_created');
        $this->payment_success = $this->get_option('payment_success');
        $this->payment_failed = $this->get_option('payment_failed');
        $this->payment_notice_successful = $this->get_option('payment_notice_successful');

        // Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'check_ipn_response'));
        add_action('woocommerce_thankyou', array($this, 'pay2s_payment_results'), 4);
        add_action('wp_enqueue_scripts', array($this, 'plugin_assets'), 1);
    }

    public function init_form_fields() {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Bật/Tắt', 'woo-pay2s'),
                'type' => 'checkbox',
                'label' => __('Bật phương thức thanh toán', 'woo-pay2s'),
                'default' => 'yes',
            ),
            'title' => array(
                'title' => __('Tiêu đề', 'woo-pay2s'),
                'type' => 'text',
                'description' => __('Tên phương thức thanh toán.', 'woo-pay2s'),
                'default' => __('Thanh toán trực tuyến qua Pay2S', 'woo-pay2s'),
                'desc_tip' => true,
            ),
            'description' => array(
                'title' => __('Mô tả', 'woo-pay2s'),
                'type' => 'textarea',
                'description' => __('Mô tả phương thức thanh toán hiển thị cho khách hàng.', 'woo-pay2s'),
                'default' => __('Thực hiện thanh toán trực tuyến qua Pay2S.', 'woo-pay2s'),
                'desc_tip' => true,
            ),
            'order_desc' => array(
                'title' => __('Nội dung thanh toán', 'woo-pay2s'),
                'type' => 'text',
                'description' => __('Nội dung thanh toán giúp nhận biết thanh toán cho đơn hàng nào. Sử dụng {{orderid}} để chèn mã đơn hàng.', 'woo-pay2s'),
                'default' => __('DH{{orderid}}', 'woo-pay2s'),
                'desc_tip' => false,
            ),
            'order_created' => array(
                'title' => __('Trạng thái đơn hàng khi đơn hàng được tạo', 'woo-pay2s'),
                'type' => 'select',
                'options' => array(
                    'pending' => __('Chờ thanh toán', 'woocommerce'),
                    'processing' => __('Đang xử lý', 'woocommerce'),
                    'on-hold' => __('Tạm giữ', 'woocommerce'),
                    'completed' => __('Đã hoàn thành', 'woocommerce'),
                    'cancelled' => __('Đã hủy', 'woocommerce'),
                    'refunded' => __('Đã hoàn lại tiền', 'woocommerce'),
                    'failed' => __('Thất bại', 'woocommerce'),
                ),
                'default' => 'pending',
                'desc_tip' => true,
            ),
            'payment_success' => array(
                'title' => __('Trạng thái đơn hàng khi thanh toán thành công', 'woo-pay2s'),
                'type' => 'select',
                'options' => array(
                    'pending' => __('Chờ thanh toán', 'woocommerce'),
                    'processing' => __('Đang xử lý', 'woocommerce'),
                    'on-hold' => __('Tạm giữ', 'woocommerce'),
                    'completed' => __('Đã hoàn thành', 'woocommerce'),
                    'cancelled' => __('Đã hủy', 'woocommerce'),
                    'refunded' => __('Đã hoàn lại tiền', 'woocommerce'),
                    'failed' => __('Thất bại', 'woocommerce'),
                ),
                'default' => 'processing',
                'desc_tip' => true,
            ),
            'payment_failed' => array(
                'title' => __('Trạng thái đơn hàng khi thanh toán thất bại', 'woo-pay2s'),
                'type' => 'select',
                'options' => array(
                    'pending' => __('Chờ thanh toán', 'woocommerce'),
                    'processing' => __('Đang xử lý', 'woocommerce'),
                    'on-hold' => __('Tạm giữ', 'woocommerce'),
                    'completed' => __('Đã hoàn thành', 'woocommerce'),
                    'cancelled' => __('Đã hủy', 'woocommerce'),
                    'refunded' => __('Đã hoàn lại tiền', 'woocommerce'),
                    'failed' => __('Thất bại', 'woocommerce'),
                ),
                'default' => 'cancelled',
                'desc_tip' => true,
            ),
            'payment_notice_successful' => array(
                'title' => __('Thông báo khi thanh toán thành công', 'woo-pay2s'),
                'type' => 'textarea',
                'description' => __('Sử dụng {{orderid}} để chèn mã đơn hàng, {{amount}} để chèn số tiền.', 'woo-pay2s'),
                'default' => __('Quý khách đã thanh toán {{amount}} thành công cho đơn hàng {{orderid}}. Xin cảm ơn quý khách!', 'woo-pay2s'),
                'desc_tip' => false,
            ),
            'button_label' => array(
                'title' => __('Nút thanh toán', 'woo-pay2s'),
                'type' => 'text',
                'description' => __('Tên hiển thị trên nút thanh toán.', 'woo-pay2s'),
                'default' => __('Thanh toán với Pay2S', 'woo-pay2s'),
                'desc_tip' => true,
            ),
            'bankAccounts' => [
                'type' => 'bankAccounts',
            ],
            'partner_code' => array(
                'title' => __('Partner Code', 'woo-pay2s'),
                'type' => 'text',
                'description' => __('Được cung cấp bởi Pay2S.', 'woo-pay2s'),
                'desc_tip' => false,
            ),
            'access_key' => array(
                'title' => __('Access Key', 'woo-pay2s'),
                'type' => 'password',
                'description' => __('Được cung cấp bởi Pay2S.', 'woo-pay2s'),
                'desc_tip' => false,
            ),
            'secret_key' => array(
                'title' => __('Secret Key', 'woo-pay2s'),
                'type' => 'password',
                'description' => __('Được cung cấp bởi Pay2S.', 'woo-pay2s'),
                'desc_tip' => false,
            ),
            'api_endpoint' => array(
                'title' => __('API Endpoint', 'woo-pay2s'),
                'type' => 'text',
                'description' => __('API Endpoint cho Pay2S. Ví dụ: https://payment.pay2s.vn/v1/gateway/api/create', 'woo-pay2s'),
                'default' => __('https://payment.pay2s.vn/v1/gateway/api/create', 'woo-pay2s'),
                'desc_tip' => false,
            ),
        );
    }

    public function plugin_assets() {
        wp_enqueue_style('woo-pay2s-style', WOO_PAY2S_PLUGIN_URL . 'assets/css/woo-pay2s.css', array(), '1.0.0', 'all');
    }

    public function generate_bankAccounts_html() {
        ob_start(); ?>
        <tr>
            <th scope="row" class="titledesc"><?php esc_html_e('Tài khoản ngân hàng:', 'woocommerce'); ?></th>
            <td class="forminp" id="bacs_accounts">
                <div class="wc_input_table_wrapper">
                    <table class="widefat wc_input_table sortable" cellspacing="0">
                        <thead>
                        <tr>
                            <th class="sort">&nbsp;</th>
                            <th><?php esc_html_e('Số tài khoản', 'woocommerce'); ?></th>
                            <th><?php esc_html_e('BankCode', 'woocommerce'); ?></th>
                        </tr>
                        </thead>
                        <tbody class="accounts">
                        <?php
                        $i = -1;
                        if ($this->bankAccounts) {
                            foreach ($this->bankAccounts as $account) {
                                $i++;
                                echo '<tr class="account">
                                    <td class="sort"></td>
                                    <td><input type="text" value="'.esc_attr($account['account_number']).'" name="bacs_account_number['.esc_attr($i).']" /></td>
                                    <td><input type="text" value="'.esc_attr(wp_unslash($account['bank_id'])).'" name="bacs_bank_id['.esc_attr($i).']" /></td>
                                </tr>';
                            }
                        } ?>
                        </tbody>
                        <tfoot>
                        <tr>
                            <th colspan="7">
                                <a href="#" class="add button"><?php esc_html_e('+ Thêm tài khoản', 'woocommerce'); ?></a>
                                <a href="#" class="remove_rows button"><?php esc_html_e('Xóa tài khoản', 'woocommerce'); ?></a>
                            </th>
                        </tr>
                        </tfoot>
                    </table>
                </div>
                <script type="text/javascript">
                    jQuery(function () {
                        jQuery('#bacs_accounts').on('click', 'a.add', function () {
                            var size = jQuery('#bacs_accounts').find('tbody .account').length;
                            jQuery('<tr class="account">\
                                <td class="sort"></td>\
                                <td><input type="text" name="bacs_account_number[' + size + ']" /></td>\
                                <td><input type="text" name="bacs_bank_id[' + size + ']" /></td>\
                            </tr>').appendTo('#bacs_accounts table tbody');
                            return false;
                        });

                        jQuery('#bacs_accounts').on('click', 'a.remove_rows', function () {
                            jQuery('#bacs_accounts').find('tbody .account:last').remove();
                            return false;
                        });
                    });
                </script>
            </td>
        </tr>
        <tr>
            <td></td>
            <td>
                Để biết dữ liệu ngân hàng cho cột Bank Code, vui lòng tham khảo thêm: <a hrefhttps://docs.pay2s.vn/api/bankcode.htmlks" title="xem định danh ngân hàng" target="_blank">https://docs.pay2s.vn/api/bankcode.html</a>
            </td>
        </tr>
        <?php
        return ob_get_clean();
    }

    public function updateBankAccounts() {
        $accounts = [];

        if (isset($_POST['bacs_account_number'], $_POST['bacs_bank_id'])) {
            $account_numbers = wc_clean(wp_unslash($_POST['bacs_account_number']));
            $bank_ids = wc_clean(wp_unslash($_POST['bacs_bank_id']));

            foreach ($account_numbers as $i => $number) {
                if (!isset($number)) {
                    continue;
                }

                $accounts[] = [
                    'account_number' => $number,
                    'bank_id' => $bank_ids[$i]
                ];
            }
        }

        update_option('woocommerce_bacs_accounts', $accounts);
    }

    public function process_admin_options() {
        parent::process_admin_options();
        $this->updateBankAccounts();
    }

    public function pay2s_payment_results($order_id) {
        //  echo "hi";
        //  if (!empty($_GET)) {

        // Sanitize input data
        $partnerCode = sanitize_text_field($_GET["partnerCode"] ?? '');
        $orderId = sanitize_text_field($_GET["orderId"] ?? '');
        $message = sanitize_text_field($_GET["message"] ?? '');
        $transId = sanitize_text_field($_GET["transId"] ?? '');
        $orderInfo = sanitize_text_field($_GET["orderInfo"] ?? '');
        $amount = sanitize_text_field($_GET["amount"] ?? '');
        $resultCode = sanitize_text_field($_GET["resultCode"] ?? '');
        $responseTime = sanitize_text_field($_GET["responseTime"] ?? '');
        $requestId = sanitize_text_field($_GET["requestId"] ?? '');
        $extraData = sanitize_text_field($_GET["extraData"] ?? '');
        $payType = sanitize_text_field($_GET["payType"] ?? '');
        $orderType = sanitize_text_field($_GET["orderType"] ?? '');
        $m2signature = sanitize_text_field($_GET["m2signature"] ?? ''); // Pay2S signature

        // Get order
        $order = wc_get_order($order_id);
        $accessKey = $this->access_key;
        $secretKey = $this->secret_key;

        // Generate rawHash and signature
        $rawHash = "accessKey=$accessKey&amount=$amount&message=$message&orderId=$orderId&orderInfo=$orderInfo&orderType=$orderType&partnerCode=$partnerCode&payType=$payType&requestId=$requestId&responseTime=$responseTime&resultCode=$resultCode";
        $partnerSignature = hash_hmac("sha256", $rawHash, $secretKey);

        if ($m2signature === $partnerSignature) {
            if ($resultCode == '0') {
                $array = array(
                    '{{orderid}}' => $order_id,
                    '{{amount}}' => number_format($amount, '0', '.', '.') . 'đ'
                );
                $payment_notice_successful = strtr($this->get_option('payment_notice_successful'), $array);
                ?>
                <section class="pay2s-payment-gateway-wrapper">
                    <div class="pay2s-payment-gateway-notification">
                        <img src="<?php echo esc_url(WOO_PAY2S_PLUGIN_URL . 'assets/images/correct.svg'); ?>" alt="Transaction successful">
                        <h2 class="notification-title">Giao dịch thành công</h2>
                        <p class="notification-content"><?php echo wpautop(esc_html($payment_notice_successful)); ?></p>
                    </div>
                </section>
                <?php
            } else {
                ?>
                <section class="pay2s-payment-gateway-wrapper">
                    <div class="pay2s-payment-gateway-notification">
                        <img src="<?php echo esc_url(WOO_PAY2S_PLUGIN_URL . 'assets/images/caution.svg'); ?>" alt="Warning">
                        <h2 class="notification-title">Giao dịch thất bại</h2>
                        <p class="notification-content"><?php echo esc_html($message); ?></p>
                    </div>
                </section>
                <?php
            }
        } else {
            ?>
            <section class="pay2s-payment-gateway-wrapper">
                <div class="pay2s-payment-gateway-notification">
                    <img src="<?php echo esc_url(WOO_PAY2S_PLUGIN_URL . 'assets/images/caution.svg'); ?>" alt="Warning">
                    <h2 class="notification-title">Chữ ký không hợp lệ</h2>
                    <p class="notification-content"><?php echo esc_html($message); ?></p>
                </div>
            </section>
            <?php
        }
    }

    public function process_payment($order_id) {
        global $woocommerce;

        $order = new WC_Order($order_id);

        $partnerCode = $this->partner_code;
        $accessKey = $this->access_key;
        $secretKey = $this->secret_key;
        $endpoint = $this->endpoint;
        $requestType = "pay2s";
        $bankAccounts = $this->bankAccounts;
        $requestId = time();
        $orderInfo = str_replace('{{orderid}}', $order_id, $this->order_desc);
        $amount = $order->get_total();
        $redirectUrl = $this->get_return_url($order);
        $ipnUrl = home_url('/wc-api/WC_Gateway_Pay2S/');

        $rawHash = "accessKey=" . $accessKey . "&amount=" . $amount . "&bankAccounts=" . $bankAccounts . "&ipnUrl=" . $ipnUrl . "&orderId=" . $order_id . "&orderInfo=" . $orderInfo . "&partnerCode=" . $partnerCode . "&redirectUrl=" . $redirectUrl . "&requestId=" . $requestId . "&requestType=" . $requestType;
        $signature = hash_hmac("sha256", $rawHash, $secretKey);

        $data = array(
            'accessKey' => $accessKey,
            'partnerCode' => $partnerCode,
            'partnerName' => get_bloginfo('name'),
            'requestId' => $requestId,
            'amount' => $amount,
            'orderId' => $order_id,
            'orderInfo' => $orderInfo,
            'orderType' => $requestType,
            'bankAccounts' => $bankAccounts,
            'redirectUrl' => $redirectUrl,
            'ipnUrl' => $ipnUrl,
            'requestType' => $requestType,
            'signature' => $signature,
        );

        $response = $this->send_request($endpoint, $data);
        if ($response && isset($response->payUrl)) {
            return array(
                'result' => 'success',
                'redirect' => $response->payUrl,
            );
        } else {
            wc_add_notice(__('Failed to create payment URL.', 'woo-pay2s'), 'error');
            return array(
                'result' => 'failure',
                'redirect' => '',
            );
        }
    }

    private function send_request($url, $data) {
        $response = wp_remote_post($url, array(
            'method' => 'POST',
            'body' => json_encode($data),
            'headers' => array(
                'Content-Type' => 'application/json',
            ),
        ));
        return json_decode(wp_remote_retrieve_body($response));
    }

    public function check_ipn_response() {
        global $woocommerce;
        // Nhận dữ liệu JSON từ php://input
        $rawData = file_get_contents('php://input');
        $data = json_decode($rawData, true);

        if (is_null($data)) {
            echo 'Invalid JSON data.';
            exit;
        }

        // Tách chữ ký ra khỏi dữ liệu
        $signature = $data['m2signature'];
        // unset($data['m2signature']);
        
        // Ghi log để kiểm tra dữ liệu nhận được
        $logFile = plugin_dir_path(__FILE__) . 'pay2s_ipn.log';
        file_put_contents($logFile, date('Y-m-d H:i:s') . " - Raw data: " . print_r($data, true) . PHP_EOL, FILE_APPEND);

        // Lấy các tham số cụ thể từ dữ liệu
        $accessKey = $data['accessKey'];
        $secretKey = $this->secret_key;
        $amount = $data['amount'];
        $extraData = isset($data['extraData']) ? $data['extraData'] : '';
        $message = $data['message'];
        $orderId = $data['orderId'];
        $orderInfo = $data['orderInfo'];
        $orderType = $data['orderType'];
        $partnerCode = $data['partnerCode'];
        $payType = $data['payType'];
        $requestId = $data['requestId'];
        $responseTime = $data['responseTime'];
        $resultCode = $data['resultCode'];
        $transId = $data['transId'];

        // Tạo chuỗi rawHash
        $rawHash = "accessKey=$accessKey&amount=$amount&extraData=$extraData&message=$message&orderId=$orderId&orderInfo=$orderInfo&orderType=$orderType&partnerCode=$partnerCode&payType=$payType&requestId=$requestId&responseTime=$responseTime&resultCode=$resultCode&transId=$transId";

        // Tính toán chữ ký
        //$trimmedRawHash = trim($rawHash);
        $calculatedSignature = hash_hmac('sha256', $rawHash, $secretKey);
        

        // So sánh chữ ký và xử lý đơn hàng nếu hợp lệ
        if ($signature == $calculatedSignature) {
            $order = wc_get_order($orderId);
            if ($order) {
                if ($resultCode == '0') {
                    if ($order->get_status() != 'completed') {
                        $order->payment_complete($transId);
                        $order->update_status($this->payment_success);
                        wc_reduce_stock_levels($orderId);
                        $array = array(
                            '{{orderid}}' => $orderId,
                            '{{amount}}' => number_format($amount, '0', '.', '.') . 'đ'
                        );
                        $payment_notice_successful = strtr($this->get_option('payment_notice_successful'), $array);
                        $order->add_order_note(
                            sprintf($payment_notice_successful, $order->get_id(), wc_price($amount))
                        );
                        // Lưu trạng thái thanh toán vào meta data của đơn hàng
                        update_post_meta($orderId, '_pay2s_payment_status', 'success');
                        update_post_meta($orderId, '_pay2s_payment_message', 'Giao dịch thành công');
                    }
                    echo json_encode(['message' => 'Received payment result success']);
                    http_response_code(200);
                } else {
                    // Lưu trạng thái thanh toán thất bại vào meta data của đơn hàng
                    update_post_meta($orderId, '_pay2s_payment_status', 'failed');
                    update_post_meta($orderId, '_pay2s_payment_message', 'Thanh toán thất bại');

                    echo json_encode(['message' => 'Payment failed.']);
                    http_response_code(400);
                }
            } else {
                echo json_encode(['message' => 'Order not found.']);
                http_response_code(404);
            }
        } else {
            echo json_encode(['message' => 'Invalid signature.']);
            http_response_code(400);
        }
        exit;
    }
}
