<?php
/**
 * Plugin Name: Pay2S Payment Gateway for Woo
 * Description: Tích hợp cổng thanh toán Pay2S với WooCommerce.
 * Version: 1.0
 * Author: Pay2S
 * Author URI: https://pay2s.vn/
 * License: GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: woo-pay2s-gateway
 * Domain Path: /languages
 */


if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Define constants
define('WC_PAY2S_GATEWAY_PLUGIN_DIR', plugin_dir_path(__FILE__));

// Check if WooCommerce is active
add_action('plugins_loaded', 'wc_pay2s_gateway_init', 11);

function wc_pay2s_gateway_init() {
    if (!class_exists('WC_Payment_Gateway')) {
        return;
    }

    // Include necessary files
    require_once WC_PAY2S_GATEWAY_PLUGIN_DIR . 'includes/class-wc-gateway-pay2s.php';

    // Add the gateway to WooCommerce
    function wc_add_pay2s_gateway($methods) {
        $methods[] = 'WC_Gateway_Pay2S';
        return $methods;
    }
    add_filter('woocommerce_payment_gateways', 'wc_add_pay2s_gateway');

    // Load text domain
    function wc_pay2s_gateway_load_textdomain() {
        load_plugin_textdomain('woo-pay2s', false, dirname(plugin_basename(__FILE__)) . '/languages');
    }
    add_action('init', 'wc_pay2s_gateway_load_textdomain');

    // Activation and deactivation hooks
    register_activation_hook(__FILE__, 'wc_pay2s_gateway_activate');
    register_deactivation_hook(__FILE__, 'wc_pay2s_gateway_deactivate');

    function wc_pay2s_gateway_activate() {
        // Update plugin version in the database
        update_option('wc_pay2s_version', '1.0');
    }

    function wc_pay2s_gateway_deactivate() {
        // Clean up if necessary
    }

    add_action('plugins_loaded', 'wc_pay2s_check_update');

    function wc_pay2s_check_update() {
        $current_version = get_option('wc_pay2s_version', '1.0');

        if (version_compare($current_version, '1.0', '<')) {
            // Perform necessary updates for version 1.0
            update_option('wc_pay2s_version', '1.0');
        }
    }
    
    require_once plugin_dir_path(__FILE__) . 'includes/plugin-update-checker/plugin-update-checker.php';
	
  	if (class_exists('Puc_v4_Factory')) {
    $myUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
        'https://package.pay2s.vn/woocommerce/license/?action=get_metadata&slug=woo-pay2s',
        __FILE__,
        'woo-pay2s'
    );
	}
}

// WooCommerce is not active
add_action('admin_notices', 'wc_pay2s_gateway_woocommerce_notice');

function wc_pay2s_gateway_woocommerce_notice() {
    if (!class_exists('WooCommerce')) {
        echo '<div class="error"><p>' . __('WooCommerce Pay2S Gateway requires WooCommerce to be active.', 'woo-pay2s') . '</p></div>';
    }
}

add_filter('plugin_action_links_' . plugin_basename(__FILE__), function ($links) {
    $actionLinks = [
        'more_info' => sprintf('<a href="https://pay2s.vn" target="_blank" style="color: #e64a19; font-weight: bold; font-size: 108%%;" title="%s">%s</a>', __('More info'), __('More info')),
        'settings' => sprintf('<a href="%s" title="%s">%s</a>', admin_url('admin.php?page=wc-settings&tab=checkout&section=pay2s'), __('Thiết lập'), __('Thiết lập')),
    ];

    return array_merge($actionLinks, $links);
});
